/**
 * Configurable options to change persistence behavior
 *
 * @group Options
 */
export type PersistOptions<T> = {
    /** Default/Initial value if undefined */
    default?: T;
    /** Storage implementation, defaults to LocalStorage */
    storage?: Storage;
    /** Force value to have [proto]type */
    type?: any;
};
/**
 * Sync variable's value with persistent storage (LocalStorage by default)
 *
 * @example
 * ```ts
 * const theme = new Persist('theme.current', {default: 'os'});
 * console.log(theme.value); // Output: os
 *
 * theme.value = 'light'; // Any changes to `.value` will automatically sync with localStorage
 *
 * location.reload(); // Simulate refresh
 * console.log(theme.value); // Output: light
 * ```
 */
export declare class Persist<T> {
    readonly key: string;
    options: PersistOptions<T>;
    /** Backend service to store data, must implement `Storage` interface */
    private readonly storage;
    /** Listeners which should be notified on changes */
    private watches;
    /** Private value field */
    private _value;
    /** Current value or default if undefined */
    get value(): T;
    /** Set value with proxy object wrapper to sync future changes */
    set value(v: T | undefined);
    /**
     * @param {string} key Primary key value will be stored under
     * @param {PersistOptions<T>} options Configure using {@link PersistOptions}
     */
    constructor(key: string, options?: PersistOptions<T>);
    /** Notify listeners of change */
    private notify;
    /** Delete value from storage */
    clear(): void;
    /** Save current value to storage */
    save(): void;
    /** Load value from storage */
    load(): void;
    /**
     * Callback function which is run when there are changes
     *
     * @param {(value: T) => any} fn Callback will run on each change; it's passed the next value & it's return is ignored
     * @returns {() => void} Function which will unsubscribe the watch/callback when called
     */
    watch(fn: (value: T) => any): () => void;
    /**
     * Return value as JSON string
     *
     * @returns {string} Stringified object as JSON
     */
    toString(): string;
    /**
     * Return current value
     *
     * @returns {T} Current value
     */
    valueOf(): T;
}
/**
 * Sync class property with persistent storage (LocalStorage by default)
 *
 * @example
 * ```ts
 * class ThemeEngine {
 *   @persist({default: 'os'}) current!: string;
 * }
 *
 * const theme = new ThemeEngine();
 * console.log(theme.current) // Output: os
 *
 * theme.current = 'light'; //Any changes will be automatically saved to localStorage
 *
 * location.reload(); // Simulate refresh
 * console.log(theme.current) // Output: light
 * ```
 *
 * @group Decorators
 * @param {PersistOptions<T> & {key?: string}} options Configure using {@link PersistOptions}
 * @returns Decorator function
 */
export declare function persist<T>(options?: PersistOptions<T> & {
    key?: string;
}): (target: any, prop: any) => void;
//# sourceMappingURL=persist.d.ts.map