"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.persist = exports.Persist = void 0;
/**
 * Sync variable's value with persistent storage (LocalStorage by default)
 *
 * @example
 * ```ts
 * const theme = new Persist('theme.current', {default: 'os'});
 * console.log(theme.value); // Output: os
 *
 * theme.value = 'light'; // Any changes to `.value` will automatically sync with localStorage
 *
 * location.reload(); // Simulate refresh
 * console.log(theme.value); // Output: light
 * ```
 */
class Persist {
    key;
    options;
    /** Backend service to store data, must implement `Storage` interface */
    storage;
    /** Listeners which should be notified on changes */
    watches = {};
    /** Private value field */
    _value;
    /** Current value or default if undefined */
    get value() { return this._value !== undefined ? this._value : this.options?.default; }
    /** Set value with proxy object wrapper to sync future changes */
    set value(v) {
        if (v == null || typeof v != 'object')
            this._value = v;
        // @ts-ignore
        else
            this._value = new Proxy(v, {
                get: (target, p) => {
                    const f = typeof target[p] == 'function';
                    if (!f)
                        return target[p];
                    return (...args) => {
                        const value = target[p](...args);
                        this.save();
                        return value;
                    };
                },
                set: (target, p, newValue) => {
                    target[p] = newValue;
                    this.save();
                    return true;
                }
            });
        this.save();
    }
    /**
     * @param {string} key Primary key value will be stored under
     * @param {PersistOptions<T>} options Configure using {@link PersistOptions}
     */
    constructor(key, options = {}) {
        this.key = key;
        this.options = options;
        this.storage = options.storage || localStorage;
        this.load();
    }
    /** Notify listeners of change */
    notify(value) {
        Object.values(this.watches).forEach(watch => watch(value));
    }
    /** Delete value from storage */
    clear() {
        this.storage.removeItem(this.key);
    }
    /** Save current value to storage */
    save() {
        if (this._value === undefined)
            this.clear();
        else
            this.storage.setItem(this.key, JSON.stringify(this._value));
        this.notify(this.value);
    }
    /** Load value from storage */
    load() {
        if (this.storage[this.key] != undefined) {
            let value = JSON.parse(this.storage.getItem(this.key));
            if (value != null && typeof value == 'object' && this.options.type)
                value.__proto__ = this.options.type.prototype;
            this.value = value;
        }
        else
            this.value = this.options.default || undefined;
    }
    /**
     * Callback function which is run when there are changes
     *
     * @param {(value: T) => any} fn Callback will run on each change; it's passed the next value & it's return is ignored
     * @returns {() => void} Function which will unsubscribe the watch/callback when called
     */
    watch(fn) {
        const index = Object.keys(this.watches).length;
        this.watches[index] = fn;
        return () => { delete this.watches[index]; };
    }
    /**
     * Return value as JSON string
     *
     * @returns {string} Stringified object as JSON
     */
    toString() { return JSON.stringify(this.value); }
    /**
     * Return current value
     *
     * @returns {T} Current value
     */
    valueOf() { return this.value; }
}
exports.Persist = Persist;
/**
 * Sync class property with persistent storage (LocalStorage by default)
 *
 * @example
 * ```ts
 * class ThemeEngine {
 *   @persist({default: 'os'}) current!: string;
 * }
 *
 * const theme = new ThemeEngine();
 * console.log(theme.current) // Output: os
 *
 * theme.current = 'light'; //Any changes will be automatically saved to localStorage
 *
 * location.reload(); // Simulate refresh
 * console.log(theme.current) // Output: light
 * ```
 *
 * @group Decorators
 * @param {PersistOptions<T> & {key?: string}} options Configure using {@link PersistOptions}
 * @returns Decorator function
 */
function persist(options) {
    return (target, prop) => {
        const key = options?.key || `${target.constructor.name}.${prop.toString()}`;
        const wrapper = new Persist(key, options);
        Object.defineProperty(target, prop, {
            get: function () { return wrapper.value; },
            set: function (v) { wrapper.value = v; }
        });
    };
}
exports.persist = persist;
